import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import { Subscribable } from './subscribable';
import { isServer } from './utils';

var OnlineManager = /*#__PURE__*/function (_Subscribable) {
  _inheritsLoose(OnlineManager, _Subscribable);

  function OnlineManager() {
    return _Subscribable.apply(this, arguments) || this;
  }

  var _proto = OnlineManager.prototype;

  _proto.onSubscribe = function onSubscribe() {
    if (!this.removeHandler) {
      this.setDefaultHandler();
    }
  };

  _proto.setHandler = function setHandler(init) {
    var _this = this;

    if (this.removeHandler) {
      this.removeHandler();
    }

    this.removeHandler = init(function () {
      _this.onOnline();
    });
  };

  _proto.onOnline = function onOnline() {
    this.listeners.forEach(function (listener) {
      listener();
    });
  };

  _proto.isOnline = function isOnline() {
    return navigator.onLine === undefined || navigator.onLine;
  };

  _proto.setDefaultHandler = function setDefaultHandler() {
    var _window;

    if (!isServer && ((_window = window) == null ? void 0 : _window.addEventListener)) {
      this.setHandler(function (onOnline) {
        // Listen to online
        window.addEventListener('online', onOnline, false);
        return function () {
          // Be sure to unsubscribe if a new handler is set
          window.removeEventListener('online', onOnline);
        };
      });
    }
  };

  return OnlineManager;
}(Subscribable);

export var onlineManager = new OnlineManager();