"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.focusManager = void 0;

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _subscribable = require("./subscribable");

var _utils = require("./utils");

var FocusManager = /*#__PURE__*/function (_Subscribable) {
  (0, _inheritsLoose2.default)(FocusManager, _Subscribable);

  function FocusManager() {
    return _Subscribable.apply(this, arguments) || this;
  }

  var _proto = FocusManager.prototype;

  _proto.onSubscribe = function onSubscribe() {
    if (!this.removeHandler) {
      this.setDefaultHandler();
    }
  };

  _proto.setHandler = function setHandler(init) {
    var _this = this;

    if (this.removeHandler) {
      this.removeHandler();
    }

    this.removeHandler = init(function () {
      _this.onFocus();
    });
  };

  _proto.onFocus = function onFocus() {
    this.listeners.forEach(function (listener) {
      listener();
    });
  };

  _proto.isFocused = function isFocused() {
    // document global can be unavailable in react native
    if (typeof document === 'undefined') {
      return true;
    }

    return [undefined, 'visible', 'prerender'].includes(document.visibilityState);
  };

  _proto.setDefaultHandler = function setDefaultHandler() {
    var _window;

    if (!_utils.isServer && ((_window = window) == null ? void 0 : _window.addEventListener)) {
      this.setHandler(function (onFocus) {
        // Listen to visibillitychange and focus
        window.addEventListener('visibilitychange', onFocus, false);
        window.addEventListener('focus', onFocus, false);
        return function () {
          // Be sure to unsubscribe if a new handler is set
          window.removeEventListener('visibilitychange', onFocus);
          window.removeEventListener('focus', onFocus);
        };
      });
    }
  };

  return FocusManager;
}(_subscribable.Subscribable);

var focusManager = new FocusManager();
exports.focusManager = focusManager;